int motorEnablePin = 10;
int motorDirectionPin1 = 8; // HIGH to go left
int motorDirectionPin2 = 7; // HIGH to go right
int sliderPositionInput = 0;
int knobTouchedInput = 1;
  
void setup() {
  pinMode(motorEnablePin, OUTPUT);
  pinMode(motorDirectionPin1, OUTPUT);
  pinMode(motorDirectionPin2, OUTPUT); 
  Serial.begin(9600);
}

int numBytesInBuffer = 0;
const int maxBytesInBuffer = 10;
char command[maxBytesInBuffer];
int commandComplete = 0;
int faderPositionTarget = 512;
int faderPositionActual = 512;
int previousError = 0;
int knobTouched = 0;

int convertCommandToNumber()
{
  int n = 0;
  for(int i = 1; i<numBytesInBuffer; i++)
  {
    n = 10*n + command[i] - '0';
  }
  return n;
}

void executeCommand()
{
  if(command[0]='m') // move the fader
  {
    faderPositionTarget = convertCommandToNumber();
  }
}

void loop() {
  while(Serial.available()>0)
  {
    int r = Serial.read();
    if(r == ';') // command complete
    {
      commandComplete = 1;        
      break;
    }
    command[numBytesInBuffer] = r;
    numBytesInBuffer++;
    if(numBytesInBuffer > maxBytesInBuffer) // Oops?
    {
      numBytesInBuffer = 0;
      break;
    }
  }            
  
  if(commandComplete)
  {
    executeCommand();
        
    // clean up the buffer
    commandComplete = 0;
    numBytesInBuffer = 0;
  }
  
  faderPositionActual = analogRead(sliderPositionInput);
  if(analogRead(knobTouchedInput) < 800)
  {
    knobTouched = 1;
  }
  else
  {
    knobTouched = 0;
  }
     
  // proportional controller with bias and clamping
  int error = faderPositionActual - faderPositionTarget;
  long speed = -5*error -10*(error<0?-1:1);
  previousError = error; // could use this for a derivative controller
  if(speed > 255)
  {
    speed = 255;
  }
  else if (speed < -255)
  {
    speed = -255;
  }
  if(knobTouched)
  {
    speed = 0;
  }
  analogWrite(motorEnablePin, abs(speed));
  
  if(speed > 0)
  {
    digitalWrite(motorDirectionPin1, LOW);
    digitalWrite(motorDirectionPin2, HIGH);      
  }
  else
  {
    digitalWrite(motorDirectionPin1, HIGH);
    digitalWrite(motorDirectionPin2, LOW);      
  }
  
  Serial.print(faderPositionActual);
  Serial.print(':');
  Serial.print(knobTouched);
  Serial.println();
}
